/**
  ******************************************************************************
  * @file    SSD1963.c
  * @author  MCD Application Team
  * @version V5.1.0
  * @date    18-January-2013
  * @brief   This file includes the LCD driver for AM-240320L8TNQW00H 
  *          (LCD_ILI9320), AM-240320LDTNQW00H (LCD_SPFD5408B)
  *          and AM240320LGTNQW00H (HX8347-D) Liquid Crystal
  *          Display Module of STM3210E-EVAL board.
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; COPYRIGHT 2013 STMicroelectronics</center></h2>
  *
  * Licensed under MCD-ST Liberty SW License Agreement V2, (the "License");
  * You may not use this file except in compliance with the License.
  * You may obtain a copy of the License at:
  *
  *        http://www.st.com/software_license_agreement_liberty_v2
  *
  * Unless required by applicable law or agreed to in writing, software 
  * distributed under the License is distributed on an "AS IS" BASIS, 
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  *
  ******************************************************************************
  */ 

/* Includes ------------------------------------------------------------------*/
#include "SSD1963.h"

/** @addtogroup Utilities
  * @{
  */ 
extern void LcdWriteRegSSD1963(unsigned int Data);
extern void LcdWriteDataSSD1963(unsigned int Date);
/** @addtogroup STM32_EVAL
  * @{
  */ 

/** @addtogroup STM3210E_EVAL
  * @{
  */

/** @defgroup STM3210E_EVAL_LCD_Private_TypesDefinitions
  * @{
  */ 
typedef struct
{
  __IO uint16_t LCD_REG;
  __IO uint16_t LCD_RAM;
} LCD_TypeDef;
/**
  * @}
  */ 


/** @defgroup STM3210E_EVAL_LCD_Private_Defines
  * @{
  */
/* Note: LCD /CS is CE4 - Bank 4 of NOR/SRAM Bank 1~4 */
#define LCD_BASE           ((uint32_t)(0x60000000 | 0x0C000000))
#define LCD                ((LCD_TypeDef *) LCD_BASE)

/**
  * @}
  */ 
unsigned int  HDP=799; //Horizontal Display Period
unsigned int  HT=1000; //Horizontal Total
unsigned int  HPS=51;  //LLINE Pulse Start Position
unsigned int  LPS=3;   //	Horizontal Display Period Start Position
unsigned char HPW=8;   //	LLINE Pulse Width


unsigned int  VDP=479;	//Vertical Display Period
unsigned int  VT=530;	//Vertical Total
unsigned int  VPS=24;	//	LFRAME Pulse Start Position
unsigned int  FPS=23;	//Vertical Display Period Start Positio
unsigned char   VPW=3;	// LFRAME Pulse Width



/** @defgroup STM3210E_EVAL_LCD_Private_Macros
  * @{
  */
#define ABS(X)  ((X) > 0 ? (X) : -(X))    
/**
  * @}
  */ 
  
/** @defgroup STM3210E_EVAL_LCD_Private_Variables
  * @{
  */ 

/**
  * @}
  */ 


/** @defgroup STM3210E_EVAL_LCD_Private_FunctionPrototypes
  * @{
  */ 
#ifndef USE_Delay
static void delay(__IO uint32_t nCount);
#endif /* USE_Delay*/

/**
  * @}
  */ 


/** @defgroup STM3210E_EVAL_LCD_Private_Functions
  * @{
  */ 

/**
  * @brief  DeInitializes the LCD.
  * @param  None
  * @retval None
  */
void LCD_DeInit(void)
{ 
  GPIO_InitTypeDef GPIO_InitStructure;

  /* LCD Display Off */
  LCD_DisplayOff();

  /* BANK 4 (of NOR/SRAM Bank 1~4) is disabled */
  FSMC_NORSRAMCmd(FSMC_Bank1_NORSRAM4, ENABLE);
  
  /* LCD_SPI DeInit */
  FSMC_NORSRAMDeInit(FSMC_Bank1_NORSRAM4);
   
  /* Set PD.00(D2), PD.01(D3), PD.04(NOE), PD.05(NWE), PD.08(D13), PD.09(D14),
     PD.10(D15), PD.14(D0), PD.15(D1) as input floating */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_4 | GPIO_Pin_5 |
                                GPIO_Pin_8 | GPIO_Pin_9 | GPIO_Pin_10 | GPIO_Pin_14 | 
                                GPIO_Pin_15;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
  GPIO_Init(GPIOD, &GPIO_InitStructure);
  /* Set PE.07(D4), PE.08(D5), PE.09(D6), PE.10(D7), PE.11(D8), PE.12(D9), PE.13(D10),
     PE.14(D11), PE.15(D12) as alternate function push pull */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9 | GPIO_Pin_10 | 
                                GPIO_Pin_11 | GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_14 | 
                                GPIO_Pin_15;
  GPIO_Init(GPIOE, &GPIO_InitStructure);
  /* Set PF.00(A0 (RS)) as alternate function push pull */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0;
  GPIO_Init(GPIOF, &GPIO_InitStructure);
  /* Set PG.12(NE4 (LCD/CS)) as alternate function push pull - CE3(LCD /CS) */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_12;
  GPIO_Init(GPIOG, &GPIO_InitStructure); 
}

/**
  * @brief  Initializes the LCD.
  * @param  None
  * @retval None
  */
void SSD1963_LCD_Init(void)
{ 
  /* Configure the LCD Control pins */
  LCD_CtrlLinesConfig();
  
  /* Configure the FSMC Parallel interface */
  LCD_FSMCConfig();

	GPIO_SetBits(GPIOD,GPIO_Pin_4);  //RD = 1 
	GPIO_SetBits(GPIOD,GPIO_Pin_5);  //WR = 1
	GPIO_ResetBits(GPIOC,GPIO_Pin_0); //RESET = 0
		delay(50);
	GPIO_SetBits(GPIOC,GPIO_Pin_0); //RESET = 1
		delay(50);
	GPIO_ResetBits(GPIOG,GPIO_Pin_12); //CS = 0	
	
  /* delay 50 ms */
		delay(50); 

		LcdWriteRegSSD1963(0x00E2);	
	LcdWriteDataSSD1963(0x0023);
	// Set PLL with OSC = 10MHz (hardware)
	// Multiplier N = 35, VCO (>250MHz)= OSC*(N+1), VCO = 360MHz	   
	LcdWriteDataSSD1963(0x0002);
	// Divider M = 2, PLL = 360/(M+1) = 120MHz
	LcdWriteDataSSD1963(0x0004);
	// Validate M and N values

		LcdWriteRegSSD1963(0x00E0);  // PLL enable
	LcdWriteDataSSD1963(0x0001);
		delay(50);
	LcdWriteRegSSD1963(0x00E0);
	LcdWriteDataSSD1963(0x0003);
		delay(50);
	LcdWriteRegSSD1963(0x0001);  // software reset
		delay(50);
		LcdWriteRegSSD1963(0x00E6);	//PLL setting for PCLK, depends on resolution
	
	//Set LSHIFT freq, i.e. the DCLK with PLL freq 120MHz set previously
	//Typical DCLK for AT070TN92 is 34MHz
	//34MHz = 120MHz*(LCDC_FPR+1)/2^20
	//LCDC_FPR = 300000 (0x0493E0)
	LcdWriteDataSSD1963(0x0004);
	LcdWriteDataSSD1963(0x0093);
	LcdWriteDataSSD1963(0x00e0);

		LcdWriteRegSSD1963(0x00B0);	//LCD SPECIFICATION
	LcdWriteDataSSD1963(0x0000);
	LcdWriteDataSSD1963(0x0000);
	LcdWriteDataSSD1963((HDP>>8)&0X00FF);  //Set HDP
	LcdWriteDataSSD1963(HDP&0X00FF);
	LcdWriteDataSSD1963((VDP>>8)&0X00FF);  //Set VDP
	LcdWriteDataSSD1963(VDP&0X00FF);
	LcdWriteDataSSD1963(0x0000);

		LcdWriteRegSSD1963(0x00B4);	//HSYNC
	LcdWriteDataSSD1963((HT>>8)&0X00FF);  //Set HT
	LcdWriteDataSSD1963(HT&0X00FF);
	LcdWriteDataSSD1963((HPS>>8)&0X00FF);  //Set HPS
	LcdWriteDataSSD1963(HPS&0X00FF);
	LcdWriteDataSSD1963(HPW);			   //Set HPW
	LcdWriteDataSSD1963((LPS>>8)&0X00FF);  //Set HPS
	LcdWriteDataSSD1963(LPS&0X00FF);
	LcdWriteDataSSD1963(0x0000);

		LcdWriteRegSSD1963(0x00B6);	//VSYNC
	LcdWriteDataSSD1963((VT>>8)&0X00FF);   //Set VT
	LcdWriteDataSSD1963(VT&0X00FF);
	LcdWriteDataSSD1963((VPS>>8)&0X00FF);  //Set VPS
	LcdWriteDataSSD1963(VPS&0X00FF);
	LcdWriteDataSSD1963(VPW);			   //Set VPW
	LcdWriteDataSSD1963((FPS>>8)&0X00FF);  //Set FPS
	LcdWriteDataSSD1963(FPS&0X00FF);

		LcdWriteRegSSD1963(0x00BA);
	LcdWriteDataSSD1963(0x0005);//LCD_WR_Data(0x000F);    //GPIO[3:0] out 1

		LcdWriteRegSSD1963(0x00B8);
	LcdWriteDataSSD1963(0x0007);    //GPIO3=input, GPIO[2:0]=output
	LcdWriteDataSSD1963(0x0001);    //GPIO0 normal

		LcdWriteRegSSD1963(0x0036); //rotation
	LcdWriteDataSSD1963(0x0000);

		LcdWriteRegSSD1963(0x00F0); //pixel data interface
	LcdWriteDataSSD1963(0x0003);

		delay(50);

	LCD_Clear(0x00ff);
	
		LcdWriteRegSSD1963(0x0026); //set_gamma_curve
	LcdWriteDataSSD1963(0x0001);

		LcdWriteRegSSD1963(0x0029); //display on

		LcdWriteRegSSD1963(0x00BE); //set PWM for B/L
	LcdWriteDataSSD1963(0x0006);
	LcdWriteDataSSD1963(0x0080);
	LcdWriteDataSSD1963(0x0001);
	LcdWriteDataSSD1963(0x00f0);
	LcdWriteDataSSD1963(0x0000);
	LcdWriteDataSSD1963(0x0000);

		LcdWriteRegSSD1963(0x00d0);//ö̬ 
	LcdWriteDataSSD1963(0x000d);
}

void LCD_Clear(uint16_t Color)
{

	uint32_t l=800,w;

		LcdWriteRegSSD1963(0x002A);	
		LcdWriteDataSSD1963(0);	    
		LcdWriteDataSSD1963(0);
		LcdWriteDataSSD1963(HDP>>8);	    
		LcdWriteDataSSD1963(HDP&0x00ff);
			LcdWriteRegSSD1963(0x002b);	
		LcdWriteDataSSD1963(0);	    
		LcdWriteDataSSD1963(0);
		LcdWriteDataSSD1963(VDP>>8);	    
		LcdWriteDataSSD1963(VDP&0x00ff);
		LcdWriteRegSSD1963(0x002c);
		
		while(l--)
		{
				for(w=0;w<480;w++) 
					LcdWriteDataSSD1963(Color);
		}  
}

/**
  * @brief  Enables the Display.
  * @param  None
  * @retval None
  */
void LCD_DisplayOn(void)
{
  /* Display On */
	LcdWriteRegSSD1963(0x0029); //display on
}

/**
  * @brief  Disables the Display.
  * @param  None
  * @retval None
  */
void LCD_DisplayOff(void)
{
  /* Display Off */
	LcdWriteRegSSD1963(0x0028); //display off
}

/**
  * @brief  Configures LCD Control lines (FSMC Pins) in alternate function mode.
  * @param  None
  * @retval None
  */
void LCD_CtrlLinesConfig(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;
  /* Enable FSMC, GPIOD, GPIOE, GPIOF, GPIOG and AFIO clocks */
  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_FSMC, ENABLE);
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC | RCC_APB2Periph_GPIOD | 
                         RCC_APB2Periph_GPIOE | RCC_APB2Periph_GPIOF | 
                         RCC_APB2Periph_GPIOG | RCC_APB2Periph_AFIO, ENABLE);
	
	/*configure PC.00 as output push-pull,connect to lcd control RESET signal*/
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_Init(GPIOC, &GPIO_InitStructure);
	
  /* Set PD.00(D2), PD.01(D3), PD.04(NOE), PD.05(NWE), PD.08(D13), PD.09(D14),
     PD.10(D15), PD.14(D0), PD.15(D1) as alternate function push pull */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_4 | GPIO_Pin_5 |
                                GPIO_Pin_8 | GPIO_Pin_9 | GPIO_Pin_10 | GPIO_Pin_14 | 
                                GPIO_Pin_15;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
  GPIO_Init(GPIOD, &GPIO_InitStructure);
  /* Set PE.07(D4), PE.08(D5), PE.09(D6), PE.10(D7), PE.11(D8), PE.12(D9), PE.13(D10),
     PE.14(D11), PE.15(D12) as alternate function push pull */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9 | GPIO_Pin_10 | 
                                GPIO_Pin_11 | GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_14 | 
                                GPIO_Pin_15;
  GPIO_Init(GPIOE, &GPIO_InitStructure);
  /* Set PF.00(A0 (RS)) as alternate function push pull */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0;
  GPIO_Init(GPIOF, &GPIO_InitStructure);
  /* Set PG.12(NE4 (LCD/CS)) as alternate function push pull - CE3(LCD /CS) */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_12;
  GPIO_Init(GPIOG, &GPIO_InitStructure);
}

/**
  * @brief  Configures the Parallel interface (FSMC) for LCD(Parallel mode)
  * @param  None
  * @retval None
  */
void LCD_FSMCConfig(void)
{
  FSMC_NORSRAMInitTypeDef  FSMC_NORSRAMInitStructure;
  FSMC_NORSRAMTimingInitTypeDef  p;
  /*-- FSMC Configuration -----------------------------------------------------*/
  /*----------------------- SRAM Bank 4 ---------------------------------------*/
  /* FSMC_Bank1_NORSRAM4 configuration */
  p.FSMC_AddressSetupTime = 1;
  p.FSMC_AddressHoldTime = 1;
  p.FSMC_DataSetupTime = 3;
  p.FSMC_BusTurnAroundDuration = 0;
  p.FSMC_CLKDivision = 1;
  p.FSMC_DataLatency = 0;
  p.FSMC_AccessMode = FSMC_AccessMode_A;
  /* Color LCD configuration
     LCD configured as follow:
        - Data/Address MUX = Disable
        - Memory Type = SRAM
        - Data Width = 16bit
        - Write Operation = Enable
        - Extended Mode = Enable
        - Asynchronous Wait = Disable */
  FSMC_NORSRAMInitStructure.FSMC_Bank = FSMC_Bank1_NORSRAM4;
  FSMC_NORSRAMInitStructure.FSMC_DataAddressMux = FSMC_DataAddressMux_Disable;
  FSMC_NORSRAMInitStructure.FSMC_MemoryType = FSMC_MemoryType_SRAM;
  FSMC_NORSRAMInitStructure.FSMC_MemoryDataWidth = FSMC_MemoryDataWidth_16b;
  FSMC_NORSRAMInitStructure.FSMC_BurstAccessMode = FSMC_BurstAccessMode_Disable;
  FSMC_NORSRAMInitStructure.FSMC_AsynchronousWait = FSMC_AsynchronousWait_Disable;  
  FSMC_NORSRAMInitStructure.FSMC_WaitSignalPolarity = FSMC_WaitSignalPolarity_Low;
  FSMC_NORSRAMInitStructure.FSMC_WrapMode = FSMC_WrapMode_Disable;
  FSMC_NORSRAMInitStructure.FSMC_WaitSignalActive = FSMC_WaitSignalActive_BeforeWaitState;
  FSMC_NORSRAMInitStructure.FSMC_WriteOperation = FSMC_WriteOperation_Enable;
  FSMC_NORSRAMInitStructure.FSMC_WaitSignal = FSMC_WaitSignal_Disable;
  FSMC_NORSRAMInitStructure.FSMC_ExtendedMode = FSMC_ExtendedMode_Disable;
  FSMC_NORSRAMInitStructure.FSMC_WriteBurst = FSMC_WriteBurst_Disable;
  FSMC_NORSRAMInitStructure.FSMC_ReadWriteTimingStruct = &p;
  FSMC_NORSRAMInitStructure.FSMC_WriteTimingStruct = &p;
  FSMC_NORSRAMInit(&FSMC_NORSRAMInitStructure);  
  /* BANK 4 (of NOR/SRAM Bank 1~4) is enabled */
  FSMC_NORSRAMCmd(FSMC_Bank1_NORSRAM4, ENABLE);
}



#ifndef USE_Delay
/**
  * @brief  Inserts a delay time.
  * @param  nCount: specifies the delay time length.
  * @retval None
  */
static void delay(vu32 nCount)
{
  vu32 index = 0; 
  for(index = (100000 * nCount); index != 0; index--)
  {
		;
  }
}
#endif /* USE_Delay */
/**
  * @}
  */ 

/**
  * @}
  */ 
  
/**
  * @}
  */ 

/**
  * @}
  */ 
  
/**
  * @}
  */  

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/






